#lang racket
(require racket/draw)

(define WIDTH 600)
(define HEIGHT 400)
(define UNIT 6) ; pixels in unit witdth square
(define OFFSET-X 140) ; starting x offset
(define OFFSET-Y 75) ; starting y offset

(define START-X (- (/ WIDTH 2)  UNIT OFFSET-X))
(define START-Y (- (/ HEIGHT 2) UNIT OFFSET-Y))

(define tiling (make-bitmap WIDTH HEIGHT)) 
(define dc (new bitmap-dc% [bitmap tiling]))

(define TILE-PEN   (new pen% [color "black"] [width 1] [style 'solid]))
(send dc set-pen TILE-PEN)

(define TILE-BRUSH (new brush% [color "yellow"] [style 'solid]))
(send dc set-brush TILE-BRUSH)

; draw a single tile at (x,y) of size s
(define (draw-tile x y s)
  (send dc draw-rectangle x y s s))

; function to compute F(n)
(define (F n)
  (define (f a b cnt)
    (if (= cnt 0) b
        (f (+ a b) a (- cnt 1))))
  (f 1 0 n))

; function to draw the tiling
(define (draw-n n)
  (let* ([fn (F n)]
         [sn (* UNIT fn)]
         [fn1 (F (sub1 n))]
         [sn1 (* UNIT fn1)]
         [n-mod-4 (remainder n 4)])
    (cond [(< n 2) #f] ; do nothing tiles already drawn
          [(= n 2) (values (+ UNIT START-X) START-Y START-X START-Y)]
          [else
           (let-values ([(x1 y1 x2 y2) (draw-n (sub1 n))])
             (let-values ([(x y)
                           (case n-mod-4
                             [(0) (values (- x1 sn) y1)]
                             [(1) (values x1 (+ y1 sn1))]
                             [(2) (values (+ x1 sn1) y2)]
                             [(3) (values x2 (- y1 sn))])])
               (draw-tile x y sn)
               (values x y x1 y1)))])))

(define (draw-tiles n)
  (draw-tile START-X START-Y UNIT)
  (draw-tile (+ UNIT START-X) START-Y UNIT)
  (draw-n n)
  (print tiling))

(draw-tiles 10)