#lang racket
(require racket/draw)

(define WIDTH 300)
(define HEIGHT 110)
(define BOTTOM-MARGIN 20)

(define RADIUS 8) ; ball radius
(define DIAMETER (* 2 RADIUS))
(define DELTA-Y (- (* DIAMETER (sin (/ pi 3)))))

(define BALL-BRUSH (new brush% [color "red"]))
(define BACKGROUND-BRUSH (new brush% [color "yellow"]))
(define BOARD-BRUSH (new brush% [color "brown"]))
(define BOARD-THICKNESS 10)
(define BOARD-Y (- HEIGHT BOARD-THICKNESS BOTTOM-MARGIN))

(define TEXT-X (+ 5 (* RADIUS 5)))
(define TEXT-Y (- HEIGHT BOTTOM-MARGIN))

; x, y location to start drawing balls
(define START-X 20)
(define START-Y (- BOARD-Y RADIUS))

(define drawing (make-bitmap WIDTH HEIGHT)) 
(define dc (new bitmap-dc% [bitmap drawing]))

(define (draw-ball x y) ; draw ball with center at (x,y)
  (send dc draw-ellipse (- x RADIUS) (- y RADIUS) DIAMETER DIAMETER))

(define (draw-pile n start-x)
  {let ([rem n]
        [x start-x]
        [y START-Y])
    (define (draw-row x y n max)
      (when (and (> rem 0) (<= n max))
        (set! rem (sub1 rem))
        (draw-ball x y)
        (draw-row (+ x DIAMETER) y (add1 n) max)))
    (for ([r (in-range 5 0 -1)])
      (draw-row x y 1 r)
      (set! x (+ x RADIUS))
      (set! y (+ y DELTA-Y)))})
  
(define pile (make-vector 3 15))
  
(define (draw-game)
  (send dc set-pen "black" 2 'solid)
  (send dc set-brush BACKGROUND-BRUSH)
  (send dc draw-rectangle 0 0 WIDTH HEIGHT)
  (send dc set-brush BOARD-BRUSH)
  (send dc draw-rectangle 0 BOARD-Y WIDTH BOARD-THICKNESS)
  (send dc set-brush BALL-BRUSH)

  (draw-pile (vector-ref pile 0) START-X)
  (send dc draw-text "0" TEXT-X TEXT-Y)
  
  (draw-pile (vector-ref pile 1) (+ START-X (* 6 DIAMETER)))
  (send dc draw-text "1" (+ TEXT-X (* 6 DIAMETER)) TEXT-Y)
  
  (draw-pile (vector-ref pile 2) (+ START-X (* 12 DIAMETER)))
  (send dc draw-text "2" (+ TEXT-X (* 12 DIAMETER)) TEXT-Y)
  (print drawing)
  )

(define nim-sum bitwise-xor)

(define (random-pile) ; select a random pile that has balls
  (let ([i (random 3)])
    (if (> (vector-ref pile i) 0) i (random-pile))))
  
(define (find-move)
  (let* ([balls (vector->list pile)]
         [s (apply nim-sum balls)])
    (if (= 0 s) ; stall
        (let ([i (random-pile)]) (values i 1))
        (let ([n (list->vector (map (λ (b) (nim-sum s b)) balls))])
          (define (test? i) (< (vector-ref n i) (vector-ref pile i)))
          (define (move i) (values i  (- (vector-ref pile i) (vector-ref n i))))  
          (cond [(test? 0) (move 0)]
                [(test? 1) (move 1)]
                [(test? 2) (move 2)])))))

(define (apply-move p n) ; remove n balls from pile p
  (vector-set! pile p (- (vector-ref pile p) n)))

(define (game-over?)
  (for/and ([i (in-range 3)]) (= 0 (vector-ref pile i))))

(define (valid-move? p n)
  (cond [(not (<= 0 p 2)) #f]
        [(< n 0) #f]
        [else (>= (vector-ref pile p) n)]))

(define (move p n)
  (if (valid-move? p n)
      (begin (apply-move p n)
             (if (game-over?)
                 (printf "\nYou win!")
                 (let-values ([(p n) (find-move)])
                   (draw-game)
                   (printf "\n\nComputer removes ~a balls from pile ~a.\n" n p)
                   (apply-move p n)
                   (draw-game)
                   (newline)
                   (newline)
                   (when (game-over?)
                     (printf "\nComputer wins!")))))
      (printf"\n Invalid move.\n\n")))

(define (init)
  (for ([i (in-range 3)]) (vector-set! pile i (random 10 16)))
  (newline)
  (draw-game)
  (newline)
  (newline))

(init)